# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/core/InfraObject.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from feast.protos.feast.core import DatastoreTable_pb2 as feast_dot_core_dot_DatastoreTable__pb2
from feast.protos.feast.core import DynamoDBTable_pb2 as feast_dot_core_dot_DynamoDBTable__pb2
from feast.protos.feast.core import SqliteTable_pb2 as feast_dot_core_dot_SqliteTable__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='feast/core/InfraObject.proto',
  package='feast.core',
  syntax='proto3',
  serialized_options=b'\n\020feast.proto.coreB\020InfraObjectProtoZ3github.com/feast-dev/feast/sdk/go/protos/feast/core',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1c\x66\x65\x61st/core/InfraObject.proto\x12\nfeast.core\x1a\x1f\x66\x65\x61st/core/DatastoreTable.proto\x1a\x1e\x66\x65\x61st/core/DynamoDBTable.proto\x1a\x1c\x66\x65\x61st/core/SqliteTable.proto\"7\n\x05Infra\x12.\n\rinfra_objects\x18\x01 \x03(\x0b\x32\x17.feast.core.InfraObject\"\xb6\x02\n\x0bInfraObject\x12\x1f\n\x17infra_object_class_type\x18\x01 \x01(\t\x12\x33\n\x0e\x64ynamodb_table\x18\x02 \x01(\x0b\x32\x19.feast.core.DynamoDBTableH\x00\x12\x35\n\x0f\x64\x61tastore_table\x18\x03 \x01(\x0b\x32\x1a.feast.core.DatastoreTableH\x00\x12/\n\x0csqlite_table\x18\x04 \x01(\x0b\x32\x17.feast.core.SqliteTableH\x00\x12;\n\x0c\x63ustom_infra\x18\x64 \x01(\x0b\x32#.feast.core.InfraObject.CustomInfraH\x00\x1a\x1c\n\x0b\x43ustomInfra\x12\r\n\x05\x66ield\x18\x01 \x01(\x0c\x42\x0e\n\x0cinfra_objectBY\n\x10\x66\x65\x61st.proto.coreB\x10InfraObjectProtoZ3github.com/feast-dev/feast/sdk/go/protos/feast/coreb\x06proto3'
  ,
  dependencies=[feast_dot_core_dot_DatastoreTable__pb2.DESCRIPTOR,feast_dot_core_dot_DynamoDBTable__pb2.DESCRIPTOR,feast_dot_core_dot_SqliteTable__pb2.DESCRIPTOR,])




_INFRA = _descriptor.Descriptor(
  name='Infra',
  full_name='feast.core.Infra',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='infra_objects', full_name='feast.core.Infra.infra_objects', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=139,
  serialized_end=194,
)


_INFRAOBJECT_CUSTOMINFRA = _descriptor.Descriptor(
  name='CustomInfra',
  full_name='feast.core.InfraObject.CustomInfra',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='field', full_name='feast.core.InfraObject.CustomInfra.field', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=463,
  serialized_end=491,
)

_INFRAOBJECT = _descriptor.Descriptor(
  name='InfraObject',
  full_name='feast.core.InfraObject',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='infra_object_class_type', full_name='feast.core.InfraObject.infra_object_class_type', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='dynamodb_table', full_name='feast.core.InfraObject.dynamodb_table', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='datastore_table', full_name='feast.core.InfraObject.datastore_table', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sqlite_table', full_name='feast.core.InfraObject.sqlite_table', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='custom_infra', full_name='feast.core.InfraObject.custom_infra', index=4,
      number=100, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_INFRAOBJECT_CUSTOMINFRA, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='infra_object', full_name='feast.core.InfraObject.infra_object',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=197,
  serialized_end=507,
)

_INFRA.fields_by_name['infra_objects'].message_type = _INFRAOBJECT
_INFRAOBJECT_CUSTOMINFRA.containing_type = _INFRAOBJECT
_INFRAOBJECT.fields_by_name['dynamodb_table'].message_type = feast_dot_core_dot_DynamoDBTable__pb2._DYNAMODBTABLE
_INFRAOBJECT.fields_by_name['datastore_table'].message_type = feast_dot_core_dot_DatastoreTable__pb2._DATASTORETABLE
_INFRAOBJECT.fields_by_name['sqlite_table'].message_type = feast_dot_core_dot_SqliteTable__pb2._SQLITETABLE
_INFRAOBJECT.fields_by_name['custom_infra'].message_type = _INFRAOBJECT_CUSTOMINFRA
_INFRAOBJECT.oneofs_by_name['infra_object'].fields.append(
  _INFRAOBJECT.fields_by_name['dynamodb_table'])
_INFRAOBJECT.fields_by_name['dynamodb_table'].containing_oneof = _INFRAOBJECT.oneofs_by_name['infra_object']
_INFRAOBJECT.oneofs_by_name['infra_object'].fields.append(
  _INFRAOBJECT.fields_by_name['datastore_table'])
_INFRAOBJECT.fields_by_name['datastore_table'].containing_oneof = _INFRAOBJECT.oneofs_by_name['infra_object']
_INFRAOBJECT.oneofs_by_name['infra_object'].fields.append(
  _INFRAOBJECT.fields_by_name['sqlite_table'])
_INFRAOBJECT.fields_by_name['sqlite_table'].containing_oneof = _INFRAOBJECT.oneofs_by_name['infra_object']
_INFRAOBJECT.oneofs_by_name['infra_object'].fields.append(
  _INFRAOBJECT.fields_by_name['custom_infra'])
_INFRAOBJECT.fields_by_name['custom_infra'].containing_oneof = _INFRAOBJECT.oneofs_by_name['infra_object']
DESCRIPTOR.message_types_by_name['Infra'] = _INFRA
DESCRIPTOR.message_types_by_name['InfraObject'] = _INFRAOBJECT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Infra = _reflection.GeneratedProtocolMessageType('Infra', (_message.Message,), {
  'DESCRIPTOR' : _INFRA,
  '__module__' : 'feast.core.InfraObject_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.Infra)
  })
_sym_db.RegisterMessage(Infra)

InfraObject = _reflection.GeneratedProtocolMessageType('InfraObject', (_message.Message,), {

  'CustomInfra' : _reflection.GeneratedProtocolMessageType('CustomInfra', (_message.Message,), {
    'DESCRIPTOR' : _INFRAOBJECT_CUSTOMINFRA,
    '__module__' : 'feast.core.InfraObject_pb2'
    # @@protoc_insertion_point(class_scope:feast.core.InfraObject.CustomInfra)
    })
  ,
  'DESCRIPTOR' : _INFRAOBJECT,
  '__module__' : 'feast.core.InfraObject_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.InfraObject)
  })
_sym_db.RegisterMessage(InfraObject)
_sym_db.RegisterMessage(InfraObject.CustomInfra)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
