# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/core/SavedDataset.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from feast.protos.feast.core import DataSource_pb2 as feast_dot_core_dot_DataSource__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='feast/core/SavedDataset.proto',
  package='feast.core',
  syntax='proto3',
  serialized_options=b'\n\020feast.proto.coreB\021SavedDatasetProtoZ3github.com/feast-dev/feast/sdk/go/protos/feast/core',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1d\x66\x65\x61st/core/SavedDataset.proto\x12\nfeast.core\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1b\x66\x65\x61st/core/DataSource.proto\"\xa5\x02\n\x10SavedDatasetSpec\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x10\n\x08\x66\x65\x61tures\x18\x03 \x03(\t\x12\x11\n\tjoin_keys\x18\x04 \x03(\t\x12\x1a\n\x12\x66ull_feature_names\x18\x05 \x01(\x08\x12\x30\n\x07storage\x18\x06 \x01(\x0b\x32\x1f.feast.core.SavedDatasetStorage\x12\x1c\n\x14\x66\x65\x61ture_service_name\x18\x08 \x01(\t\x12\x34\n\x04tags\x18\x07 \x03(\x0b\x32&.feast.core.SavedDatasetSpec.TagsEntry\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xa7\x02\n\x13SavedDatasetStorage\x12:\n\x0c\x66ile_storage\x18\x04 \x01(\x0b\x32\".feast.core.DataSource.FileOptionsH\x00\x12\x42\n\x10\x62igquery_storage\x18\x05 \x01(\x0b\x32&.feast.core.DataSource.BigQueryOptionsH\x00\x12\x42\n\x10redshift_storage\x18\x06 \x01(\x0b\x32&.feast.core.DataSource.RedshiftOptionsH\x00\x12\x44\n\x11snowflake_storage\x18\x07 \x01(\x0b\x32\'.feast.core.DataSource.SnowflakeOptionsH\x00\x42\x06\n\x04kind\"\xf7\x01\n\x10SavedDatasetMeta\x12\x35\n\x11\x63reated_timestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12:\n\x16last_updated_timestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x37\n\x13min_event_timestamp\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x37\n\x13max_event_timestamp\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"f\n\x0cSavedDataset\x12*\n\x04spec\x18\x01 \x01(\x0b\x32\x1c.feast.core.SavedDatasetSpec\x12*\n\x04meta\x18\x02 \x01(\x0b\x32\x1c.feast.core.SavedDatasetMetaBZ\n\x10\x66\x65\x61st.proto.coreB\x11SavedDatasetProtoZ3github.com/feast-dev/feast/sdk/go/protos/feast/coreb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,feast_dot_core_dot_DataSource__pb2.DESCRIPTOR,])




_SAVEDDATASETSPEC_TAGSENTRY = _descriptor.Descriptor(
  name='TagsEntry',
  full_name='feast.core.SavedDatasetSpec.TagsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='feast.core.SavedDatasetSpec.TagsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='feast.core.SavedDatasetSpec.TagsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=358,
  serialized_end=401,
)

_SAVEDDATASETSPEC = _descriptor.Descriptor(
  name='SavedDatasetSpec',
  full_name='feast.core.SavedDatasetSpec',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='feast.core.SavedDatasetSpec.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='project', full_name='feast.core.SavedDatasetSpec.project', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='features', full_name='feast.core.SavedDatasetSpec.features', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='join_keys', full_name='feast.core.SavedDatasetSpec.join_keys', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='full_feature_names', full_name='feast.core.SavedDatasetSpec.full_feature_names', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='storage', full_name='feast.core.SavedDatasetSpec.storage', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_service_name', full_name='feast.core.SavedDatasetSpec.feature_service_name', index=6,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tags', full_name='feast.core.SavedDatasetSpec.tags', index=7,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_SAVEDDATASETSPEC_TAGSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=108,
  serialized_end=401,
)


_SAVEDDATASETSTORAGE = _descriptor.Descriptor(
  name='SavedDatasetStorage',
  full_name='feast.core.SavedDatasetStorage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='file_storage', full_name='feast.core.SavedDatasetStorage.file_storage', index=0,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bigquery_storage', full_name='feast.core.SavedDatasetStorage.bigquery_storage', index=1,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='redshift_storage', full_name='feast.core.SavedDatasetStorage.redshift_storage', index=2,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='snowflake_storage', full_name='feast.core.SavedDatasetStorage.snowflake_storage', index=3,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='kind', full_name='feast.core.SavedDatasetStorage.kind',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=404,
  serialized_end=699,
)


_SAVEDDATASETMETA = _descriptor.Descriptor(
  name='SavedDatasetMeta',
  full_name='feast.core.SavedDatasetMeta',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='created_timestamp', full_name='feast.core.SavedDatasetMeta.created_timestamp', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='last_updated_timestamp', full_name='feast.core.SavedDatasetMeta.last_updated_timestamp', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='min_event_timestamp', full_name='feast.core.SavedDatasetMeta.min_event_timestamp', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max_event_timestamp', full_name='feast.core.SavedDatasetMeta.max_event_timestamp', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=702,
  serialized_end=949,
)


_SAVEDDATASET = _descriptor.Descriptor(
  name='SavedDataset',
  full_name='feast.core.SavedDataset',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='spec', full_name='feast.core.SavedDataset.spec', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='meta', full_name='feast.core.SavedDataset.meta', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=951,
  serialized_end=1053,
)

_SAVEDDATASETSPEC_TAGSENTRY.containing_type = _SAVEDDATASETSPEC
_SAVEDDATASETSPEC.fields_by_name['storage'].message_type = _SAVEDDATASETSTORAGE
_SAVEDDATASETSPEC.fields_by_name['tags'].message_type = _SAVEDDATASETSPEC_TAGSENTRY
_SAVEDDATASETSTORAGE.fields_by_name['file_storage'].message_type = feast_dot_core_dot_DataSource__pb2._DATASOURCE_FILEOPTIONS
_SAVEDDATASETSTORAGE.fields_by_name['bigquery_storage'].message_type = feast_dot_core_dot_DataSource__pb2._DATASOURCE_BIGQUERYOPTIONS
_SAVEDDATASETSTORAGE.fields_by_name['redshift_storage'].message_type = feast_dot_core_dot_DataSource__pb2._DATASOURCE_REDSHIFTOPTIONS
_SAVEDDATASETSTORAGE.fields_by_name['snowflake_storage'].message_type = feast_dot_core_dot_DataSource__pb2._DATASOURCE_SNOWFLAKEOPTIONS
_SAVEDDATASETSTORAGE.oneofs_by_name['kind'].fields.append(
  _SAVEDDATASETSTORAGE.fields_by_name['file_storage'])
_SAVEDDATASETSTORAGE.fields_by_name['file_storage'].containing_oneof = _SAVEDDATASETSTORAGE.oneofs_by_name['kind']
_SAVEDDATASETSTORAGE.oneofs_by_name['kind'].fields.append(
  _SAVEDDATASETSTORAGE.fields_by_name['bigquery_storage'])
_SAVEDDATASETSTORAGE.fields_by_name['bigquery_storage'].containing_oneof = _SAVEDDATASETSTORAGE.oneofs_by_name['kind']
_SAVEDDATASETSTORAGE.oneofs_by_name['kind'].fields.append(
  _SAVEDDATASETSTORAGE.fields_by_name['redshift_storage'])
_SAVEDDATASETSTORAGE.fields_by_name['redshift_storage'].containing_oneof = _SAVEDDATASETSTORAGE.oneofs_by_name['kind']
_SAVEDDATASETSTORAGE.oneofs_by_name['kind'].fields.append(
  _SAVEDDATASETSTORAGE.fields_by_name['snowflake_storage'])
_SAVEDDATASETSTORAGE.fields_by_name['snowflake_storage'].containing_oneof = _SAVEDDATASETSTORAGE.oneofs_by_name['kind']
_SAVEDDATASETMETA.fields_by_name['created_timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_SAVEDDATASETMETA.fields_by_name['last_updated_timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_SAVEDDATASETMETA.fields_by_name['min_event_timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_SAVEDDATASETMETA.fields_by_name['max_event_timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_SAVEDDATASET.fields_by_name['spec'].message_type = _SAVEDDATASETSPEC
_SAVEDDATASET.fields_by_name['meta'].message_type = _SAVEDDATASETMETA
DESCRIPTOR.message_types_by_name['SavedDatasetSpec'] = _SAVEDDATASETSPEC
DESCRIPTOR.message_types_by_name['SavedDatasetStorage'] = _SAVEDDATASETSTORAGE
DESCRIPTOR.message_types_by_name['SavedDatasetMeta'] = _SAVEDDATASETMETA
DESCRIPTOR.message_types_by_name['SavedDataset'] = _SAVEDDATASET
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SavedDatasetSpec = _reflection.GeneratedProtocolMessageType('SavedDatasetSpec', (_message.Message,), {

  'TagsEntry' : _reflection.GeneratedProtocolMessageType('TagsEntry', (_message.Message,), {
    'DESCRIPTOR' : _SAVEDDATASETSPEC_TAGSENTRY,
    '__module__' : 'feast.core.SavedDataset_pb2'
    # @@protoc_insertion_point(class_scope:feast.core.SavedDatasetSpec.TagsEntry)
    })
  ,
  'DESCRIPTOR' : _SAVEDDATASETSPEC,
  '__module__' : 'feast.core.SavedDataset_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.SavedDatasetSpec)
  })
_sym_db.RegisterMessage(SavedDatasetSpec)
_sym_db.RegisterMessage(SavedDatasetSpec.TagsEntry)

SavedDatasetStorage = _reflection.GeneratedProtocolMessageType('SavedDatasetStorage', (_message.Message,), {
  'DESCRIPTOR' : _SAVEDDATASETSTORAGE,
  '__module__' : 'feast.core.SavedDataset_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.SavedDatasetStorage)
  })
_sym_db.RegisterMessage(SavedDatasetStorage)

SavedDatasetMeta = _reflection.GeneratedProtocolMessageType('SavedDatasetMeta', (_message.Message,), {
  'DESCRIPTOR' : _SAVEDDATASETMETA,
  '__module__' : 'feast.core.SavedDataset_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.SavedDatasetMeta)
  })
_sym_db.RegisterMessage(SavedDatasetMeta)

SavedDataset = _reflection.GeneratedProtocolMessageType('SavedDataset', (_message.Message,), {
  'DESCRIPTOR' : _SAVEDDATASET,
  '__module__' : 'feast.core.SavedDataset_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.SavedDataset)
  })
_sym_db.RegisterMessage(SavedDataset)


DESCRIPTOR._options = None
_SAVEDDATASETSPEC_TAGSENTRY._options = None
# @@protoc_insertion_point(module_scope)
