# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/core/ValidationProfile.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from feast.protos.feast.core import SavedDataset_pb2 as feast_dot_core_dot_SavedDataset__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='feast/core/ValidationProfile.proto',
  package='feast.core',
  syntax='proto3',
  serialized_options=b'\n\020feast.proto.coreB\021ValidationProfileZ3github.com/feast-dev/feast/sdk/go/protos/feast/core',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\"feast/core/ValidationProfile.proto\x12\nfeast.core\x1a\x1d\x66\x65\x61st/core/SavedDataset.proto\"\x83\x01\n\x14GEValidationProfiler\x12\x46\n\x08profiler\x18\x01 \x01(\x0b\x32\x34.feast.core.GEValidationProfiler.UserDefinedProfiler\x1a#\n\x13UserDefinedProfiler\x12\x0c\n\x04\x62ody\x18\x01 \x01(\x0c\"0\n\x13GEValidationProfile\x12\x19\n\x11\x65xpectation_suite\x18\x01 \x01(\x0c\"\x85\x01\n\x13ValidationReference\x12)\n\x07\x64\x61taset\x18\x01 \x01(\x0b\x32\x18.feast.core.SavedDataset\x12\x37\n\x0bge_profiler\x18\x02 \x01(\x0b\x32 .feast.core.GEValidationProfilerH\x00\x42\n\n\x08profilerBZ\n\x10\x66\x65\x61st.proto.coreB\x11ValidationProfileZ3github.com/feast-dev/feast/sdk/go/protos/feast/coreb\x06proto3'
  ,
  dependencies=[feast_dot_core_dot_SavedDataset__pb2.DESCRIPTOR,])




_GEVALIDATIONPROFILER_USERDEFINEDPROFILER = _descriptor.Descriptor(
  name='UserDefinedProfiler',
  full_name='feast.core.GEValidationProfiler.UserDefinedProfiler',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='body', full_name='feast.core.GEValidationProfiler.UserDefinedProfiler.body', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=178,
  serialized_end=213,
)

_GEVALIDATIONPROFILER = _descriptor.Descriptor(
  name='GEValidationProfiler',
  full_name='feast.core.GEValidationProfiler',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='profiler', full_name='feast.core.GEValidationProfiler.profiler', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_GEVALIDATIONPROFILER_USERDEFINEDPROFILER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=82,
  serialized_end=213,
)


_GEVALIDATIONPROFILE = _descriptor.Descriptor(
  name='GEValidationProfile',
  full_name='feast.core.GEValidationProfile',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='expectation_suite', full_name='feast.core.GEValidationProfile.expectation_suite', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=215,
  serialized_end=263,
)


_VALIDATIONREFERENCE = _descriptor.Descriptor(
  name='ValidationReference',
  full_name='feast.core.ValidationReference',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='dataset', full_name='feast.core.ValidationReference.dataset', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ge_profiler', full_name='feast.core.ValidationReference.ge_profiler', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='profiler', full_name='feast.core.ValidationReference.profiler',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=266,
  serialized_end=399,
)

_GEVALIDATIONPROFILER_USERDEFINEDPROFILER.containing_type = _GEVALIDATIONPROFILER
_GEVALIDATIONPROFILER.fields_by_name['profiler'].message_type = _GEVALIDATIONPROFILER_USERDEFINEDPROFILER
_VALIDATIONREFERENCE.fields_by_name['dataset'].message_type = feast_dot_core_dot_SavedDataset__pb2._SAVEDDATASET
_VALIDATIONREFERENCE.fields_by_name['ge_profiler'].message_type = _GEVALIDATIONPROFILER
_VALIDATIONREFERENCE.oneofs_by_name['profiler'].fields.append(
  _VALIDATIONREFERENCE.fields_by_name['ge_profiler'])
_VALIDATIONREFERENCE.fields_by_name['ge_profiler'].containing_oneof = _VALIDATIONREFERENCE.oneofs_by_name['profiler']
DESCRIPTOR.message_types_by_name['GEValidationProfiler'] = _GEVALIDATIONPROFILER
DESCRIPTOR.message_types_by_name['GEValidationProfile'] = _GEVALIDATIONPROFILE
DESCRIPTOR.message_types_by_name['ValidationReference'] = _VALIDATIONREFERENCE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GEValidationProfiler = _reflection.GeneratedProtocolMessageType('GEValidationProfiler', (_message.Message,), {

  'UserDefinedProfiler' : _reflection.GeneratedProtocolMessageType('UserDefinedProfiler', (_message.Message,), {
    'DESCRIPTOR' : _GEVALIDATIONPROFILER_USERDEFINEDPROFILER,
    '__module__' : 'feast.core.ValidationProfile_pb2'
    # @@protoc_insertion_point(class_scope:feast.core.GEValidationProfiler.UserDefinedProfiler)
    })
  ,
  'DESCRIPTOR' : _GEVALIDATIONPROFILER,
  '__module__' : 'feast.core.ValidationProfile_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.GEValidationProfiler)
  })
_sym_db.RegisterMessage(GEValidationProfiler)
_sym_db.RegisterMessage(GEValidationProfiler.UserDefinedProfiler)

GEValidationProfile = _reflection.GeneratedProtocolMessageType('GEValidationProfile', (_message.Message,), {
  'DESCRIPTOR' : _GEVALIDATIONPROFILE,
  '__module__' : 'feast.core.ValidationProfile_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.GEValidationProfile)
  })
_sym_db.RegisterMessage(GEValidationProfile)

ValidationReference = _reflection.GeneratedProtocolMessageType('ValidationReference', (_message.Message,), {
  'DESCRIPTOR' : _VALIDATIONREFERENCE,
  '__module__' : 'feast.core.ValidationProfile_pb2'
  # @@protoc_insertion_point(class_scope:feast.core.ValidationReference)
  })
_sym_db.RegisterMessage(ValidationReference)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
